@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -26,9 +24,13 @@
 #ifndef __AUTHENTICATION_H__
 #define __AUTHENTICATION_H__
 
+#include <sys/cdefs.h>
+
+__BEGIN_DECLS
 extern char *groupNameForTask(int taskNum);
 extern int isAuthenticatedAsAdministrator(void);
 extern int isAuthenticatedAsAdministratorForTask(int taskNum);
 extern int isAuthenticatedAsRoot(void);
+__END_DECLS
 
 #endif /* __AUTHENTICATION_H__ */
