@@ -62,22 +62,13 @@
  * The kernel defines the format of directory entries returned by 
  * the getdirentries(2) system call.
  */
-#include <sys/types.h>
+#include <_types.h>
 #include <sys/dirent.h>
 
-#ifdef _POSIX_SOURCE
-typedef void *	DIR;
-#else
-
-#define	d_ino		d_fileno	/* backward compatibility */
-
-/* definitions for library routines operating on directories. */
-#define	DIRBLKSIZ	1024
-
-struct _telldir;		/* see telldir.h */
+struct _telldir;		/* forward reference */
 
 /* structure describing an open directory. */
-typedef struct _dirdesc {
+typedef struct {
 	int	dd_fd;		/* file descriptor associated with directory */
 	long	dd_loc;		/* offset in current buffer */
 	long	dd_size;	/* amount of data returned by getdirentries */
@@ -86,10 +77,14 @@ typedef struct _dirdesc {
 	long	dd_seek;	/* magic cookie returned by getdirentries */
 	long	dd_rewind;	/* magic cookie for rewinding */
 	int	dd_flags;	/* flags for readdir */
-	pthread_mutex_t	dd_lock; /* for thread locking */
+	__darwin_pthread_mutex_t dd_lock; /* for thread locking */
 	struct _telldir *dd_td;	/* telldir position recording */
 } DIR;
 
+#ifndef _POSIX_C_SOURCE
+/* definitions for library routines operating on directories. */
+#define	DIRBLKSIZ	1024
+
 #define	dirfd(dirp)	((dirp)->dd_fd)
 
 /* flags for opendir2 */
@@ -98,31 +93,33 @@ typedef struct _dirdesc {
 #define DTF_REWIND	0x0004	/* rewind after reading union stack */
 #define __DTF_READALL	0x0008	/* everything has been read */
 
-#ifndef NULL
-#define	NULL	0
-#endif
-
-#endif /* _POSIX_SOURCE */
+#endif /* ! _POSIX_C_SOURCE */
 
 #ifndef KERNEL
 
 #include <sys/cdefs.h>
 
 __BEGIN_DECLS
-DIR *opendir(const char *);
+#ifndef _POSIX_C_SOURCE
+int alphasort(const void *, const void *);
+#endif /* not POSIX */
+int closedir(DIR *) __DARWIN_ALIAS(closedir);
+#ifndef _POSIX_C_SOURCE
+int getdirentries(int, char *, int, long *);
+#endif /* not POSIX */
+DIR *opendir(const char *) __DARWIN_ALIAS(opendir);
+#ifndef _POSIX_C_SOURCE
+DIR *__opendir2(const char *, int) __DARWIN_ALIAS(__opendir2);
+#endif /* not POSIX */
 struct dirent *readdir(DIR *);
-void rewinddir(DIR *);
-int closedir(DIR *);
-#ifndef _POSIX_SOURCE
-DIR *__opendir2(const char *, int);
-long telldir(DIR *);
-void seekdir(DIR *, long);
+int readdir_r(DIR *, struct dirent *, struct dirent **);
+void rewinddir(DIR *) __DARWIN_ALIAS(rewinddir);
+#ifndef _POSIX_C_SOURCE
 int scandir(const char *, struct dirent ***,
     int (*)(struct dirent *), int (*)(const void *, const void *));
-int alphasort(const void *, const void *);
-int getdirentries(int, char *, int, long *);
-int readdir_r(DIR *, struct dirent *, struct dirent **);
 #endif /* not POSIX */
+void seekdir(DIR *, long) __DARWIN_ALIAS(seekdir);
+long telldir(DIR *) __DARWIN_ALIAS(telldir);
 __END_DECLS
 
 #endif /* !KERNEL */
