@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -56,21 +54,8 @@
 #include <mach/mig_errors.h>
 #include <mach/vm_statistics.h>
 
-#ifdef HOST_MACH_MSG_TRAP
-__private_extern__ kern_return_t _host_mach_msg_trap_return_ = KERN_FAILURE;
-
-#define MACH_MSG_TRAP(msg, opt, ssize, rsize, rname, to, not)	 	       \
-	((_host_mach_msg_trap_return_ == KERN_SUCCESS) ?		       \
-	 mach_msg_trap((msg), (opt), (ssize), (rsize), (rname), (to), (not)) : \
-	 mach_msg_overwrite_trap((msg), (opt), (ssize), (rsize), (rname),      \
-				 (to), (not), MACH_MSG_NULL, 0))
-
-#else
-
-#define MACH_MSG_TRAP(msg, opt, ssize, rsize, rname, to, not)	 	       \
-	 mach_msg_overwrite_trap((msg), (opt), (ssize), (rsize), (rname),      \
-				 (to), (not), MACH_MSG_NULL, 0)) 
-#endif
+#define MACH_MSG_TRAP(msg, opt, ssize, rsize, rname, to, not) \
+	 mach_msg_trap((msg), (opt), (ssize), (rsize), (rname), (to), (not))
 
 #define LIBMACH_OPTIONS	(MACH_SEND_INTERRUPT|MACH_RCV_INTERRUPT)
 
@@ -499,7 +484,7 @@ mach_msg_server(
 	kern_return_t kr;
 	mach_port_t self = mach_task_self();
 
-	options &= ~(MACH_SEND_MSG|MACH_RCV_MSG);
+	options &= ~(MACH_SEND_MSG|MACH_RCV_MSG|MACH_RCV_OVERWRITE);
 
 	reply_alloc = round_page((options & MACH_SEND_TRAILER) ? 
 			     (max_size + MAX_TRAILER_SIZE) : max_size);
@@ -589,7 +574,7 @@ mach_msg_server(
 						 MACH_SEND_MSG|MACH_RCV_MSG|MACH_SEND_TIMEOUT|options,
 						bufReply->Head.msgh_size, request_size, rcv_name,
 						MACH_MSG_TIMEOUT_NONE, MACH_PORT_NULL,
-						&bufRequest->Head, request_size);
+						&bufRequest->Head, 0);
 				}
 				
 				if ((mr != MACH_SEND_INVALID_DEST) &&
