@@ -49,7 +49,7 @@
  * Machine specific support for thread initialization
  */
 
-#if defined(__ppc__)
+#if defined(__ppc__) || defined(__ppc64__)
 #include <architecture/ppc/cframe.h>
 #endif
 
@@ -66,39 +66,46 @@ _pthread_setup(pthread_t thread,
 {
         kern_return_t r;
         unsigned int count;
+#if defined(__ppc__) || defined(__ppc64__)
 #if defined(__ppc__)
-        struct ppc_thread_state state = {0};
-	struct ppc_thread_state *ts = &state;
-
+        ppc_thread_state_t state = {0};
+	ppc_thread_state_t *ts = &state;
+	thread_state_flavor_t flavor = PPC_THREAD_STATE;
+	count = PPC_THREAD_STATE_COUNT;
+#elif defined(__ppc64__)
+        ppc_thread_state64_t state = {0};
+	ppc_thread_state64_t *ts = &state;
+	thread_state_flavor_t flavor = PPC_THREAD_STATE64;
+	count = PPC_THREAD_STATE64_COUNT;
+#endif
 	/*
 	 * Set up PowerPC registers.
 	 */
-	count = PPC_THREAD_STATE_COUNT;
 	if (suspended) {
 		PTHREAD_MACH_CALL(thread_get_state(thread->kernel_thread,
-					   PPC_THREAD_STATE,
+					   flavor,
 					   (thread_state_t) &state,
 					   &count),
 			  r);
 	}
-	ts->srr0 = (int) routine;
+	ts->srr0 = (uintptr_t)routine;
         ts->r1 = (uintptr_t)vsp - C_ARGSAVE_LEN - C_RED_ZONE;
-	ts->r3 = (int)thread;
+	ts->r3 = (uintptr_t)thread;
 	/* Incase of needresume, suspend is always set */
 	if (suspended) {
 		PTHREAD_MACH_CALL(thread_set_state(thread->kernel_thread,
-					   PPC_THREAD_STATE,
+					   flavor,
 					   (thread_state_t) &state,
-					   PPC_THREAD_STATE_COUNT),
+					   count),
 			  r);
 		if (needresume)
 			PTHREAD_MACH_CALL(thread_resume(thread->kernel_thread),
 				r);
 	} else {
 		PTHREAD_MACH_CALL(thread_create_running(mach_task_self(),
-					PPC_THREAD_STATE,
+					flavor,
 					(thread_state_t) ts,
-					PPC_THREAD_STATE_COUNT,
+					count,
 					&thread->kernel_thread),
 			r);
 	}
@@ -119,9 +126,17 @@ _pthread_setup(pthread_t thread,
 			  r);
 	}
         ts->eip = (int) routine;
-        *--sp = (int) thread;	/* argument to function */
-        *--sp = 0;		/* fake return address */
-        ts->esp = (int) sp;	/* set stack pointer */
+
+        /*
+        ** We need to simulate a 16-byte aligned stack frame as if we had
+        ** executed a call instruction. Since we're "pushing" one argument,
+        ** we need to adjust the pointer by 12 bytes (3 * sizeof (int *))
+        */
+
+        sp -= 3;              /* make sure stack is aligned */
+        *--sp = (int) thread; /* argument to function */
+        *--sp = 0;            /* fake return address */
+        ts->esp = (int) sp;   /* set stack pointer */
 	/* Incase of needresume, suspend is always set */
         if (suspended) {
 		PTHREAD_MACH_CALL(thread_set_state(thread->kernel_thread,
